@@ -79,6 +79,112 @@ rfc1123_to_iso8601_date (const gchar *date)
 	return tracker_date_format_to_iso8601 (date, RFC1123_DATE_FORMAT);
 }
 
+#if defined(HAVE_EXEMPI) && defined(PNG_iTXt_SUPPORTED)
+
+/* Handle raw profiles by Imagemagick (at least). Hex encoded with
+ * line-changes and other (undocumented/unofficial) twists.
+ */
+static gchar *
+raw_profile_new (const gchar *input,
+                 const guint  input_length,
+                 guint       *output_length)
+{
+	static const gchar* const lut = "0123456789abcdef";
+	gchar *output;
+	const gchar *ptr;
+	const gchar *length_ptr;
+	gsize size;
+	gchar *length_str;
+	guint length;
+
+	size_t len;
+	size_t i;
+	size_t o;
+	char *p;
+	char *q;
+
+	ptr = input;
+
+	if (*ptr != '\n') {
+		return NULL;
+	}
+
+	ptr++;
+
+	if (!g_ascii_isalpha (*ptr)) {
+		return NULL;
+	}
+
+	/* Skip the type string */
+	do {
+		ptr++;
+	} while (g_ascii_isalpha (*ptr));
+
+	if (*ptr != '\n') {
+		return NULL;
+	}
+
+	/* Hop over whitespaces */
+	do {
+		ptr++;
+	} while (*ptr == ' ');
+
+	if (!g_ascii_isdigit (*ptr)) {
+		return NULL;
+	}
+
+	/* Get the length string */
+	length_ptr = ptr;
+	size = 1;
+
+	do {
+		ptr++;
+		size++;
+	} while (g_ascii_isdigit (*ptr));
+
+	length_str = g_strndup (length_ptr, size - 1);
+
+	if (*ptr != '\n') {
+		return NULL;
+	}
+
+	ptr++;
+
+	length = atoi (length_str);
+	g_free (length_str);
+
+	len = length;
+	i = 0;
+	o = 0;
+
+	output = malloc (length + 1); /* A bit less with non-valid */
+
+	o = 0;
+	while (o < len) {
+		do {
+			gchar a = ptr[i];
+			p = strchr (lut, a);
+			i++;
+		} while (p == 0);
+
+		do {
+			gchar b = ptr[i];
+			q = strchr (lut, b);
+			i++;
+		} while (q == 0);
+
+		output[o] = (((p - lut) << 4) | (q - lut));
+		o++;
+	}
+
+	output[o] = '\0';
+	*output_length = o;
+
+	return output;
+}
+
+#endif /* HAVE_EXEMPI && PNG_iTXt_SUPPORTED */
+
 static void
 read_metadata (TrackerSparqlBuilder *preupdate,
                TrackerSparqlBuilder *metadata,
@@ -86,7 +192,8 @@ read_metadata (TrackerSparqlBuilder *preupdate,
                png_structp           png_ptr,
                png_infop             info_ptr,
                png_infop             end_ptr,
-               const gchar          *uri)
+               const gchar          *uri,
+               const gchar          *graph)
 {
 	MergeData md = { 0 };
 	PngData pd = { 0 };
@@ -114,21 +221,48 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 				continue;
 			}
 
-	#if defined(HAVE_EXEMPI) && defined(PNG_iTXt_SUPPORTED)
+#if defined(HAVE_EXEMPI) && defined(PNG_iTXt_SUPPORTED)
 			if (g_strcmp0 ("XML:com.adobe.xmp", text_ptr[i].key) == 0) {
 				/* ATM tracker_extract_xmp_read supports setting xd
 				 * multiple times, keep it that way as here it's
 				 * theoretically possible that the function gets
-				 * called multiple times 
+				 * called multiple times
 				 */
 				xd = tracker_xmp_new (text_ptr[i].text,
-					              text_ptr[i].itxt_length,
-					              uri);
+				                      text_ptr[i].itxt_length,
+				                      uri);
+
 				continue;
 			}
-	#endif
 
-	#if defined(HAVE_LIBEXIF) && defined(PNG_iTXt_SUPPORTED)
+			if (g_strcmp0 ("Raw profile type xmp", text_ptr[i].key) == 0) {
+				gchar *xmp_buffer;
+				guint xmp_buffer_length = 0;
+				guint input_len;
+
+				if (text_ptr[i].text_length) {
+					input_len = text_ptr[i].text_length;
+				} else {
+					input_len = text_ptr[i].itxt_length;
+				}
+
+				xmp_buffer = raw_profile_new (text_ptr[i].text,
+				                              input_len,
+				                              &xmp_buffer_length);
+
+				if (xmp_buffer) {
+					xd = tracker_xmp_new (xmp_buffer,
+					                      xmp_buffer_length,
+					                      uri);
+				}
+
+				g_free (xmp_buffer);
+
+				continue;
+			}
+#endif
+
+#if defined(HAVE_LIBEXIF) && defined(PNG_iTXt_SUPPORTED)
 			/* I'm not certain this is the key for EXIF. Using key according to
 			 * this document about exiftool:
 			 * http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/PNG.html#TextualData 
@@ -139,7 +273,7 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 					               uri);
 				continue;
 			}
-	#endif /* HAVE_LIBEXIF */
+#endif /* HAVE_LIBEXIF */
 
 			if (g_strcmp0 (text_ptr[i].key, "Author") == 0) {
 				pd.author = text_ptr[i].text;
@@ -229,11 +363,19 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", md.creator);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
+
 		tracker_sparql_builder_subject_iri (preupdate, uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nco:Contact");
 		tracker_sparql_builder_predicate (preupdate, "nco:fullname");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.creator);
+
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 
 		tracker_sparql_builder_predicate (metadata, "nco:creator");
@@ -269,6 +411,10 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		                                              md.model ? md.model : "");
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
+
 		tracker_sparql_builder_subject_iri (preupdate, equip_uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nfo:Equipment");
@@ -281,7 +427,12 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 			tracker_sparql_builder_predicate (preupdate, "nfo:model");
 			tracker_sparql_builder_object_unvalidated (preupdate, md.model);
 		}
+
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
+
 		tracker_sparql_builder_predicate (metadata, "nfo:equipment");
 		tracker_sparql_builder_object_iri (metadata, equip_uri);
 		g_free (equip_uri);
@@ -291,11 +442,19 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", md.artist);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
+
 		tracker_sparql_builder_subject_iri (preupdate, uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nco:Contact");
 		tracker_sparql_builder_predicate (preupdate, "nco:fullname");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.artist);
+
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 
 		tracker_sparql_builder_predicate (metadata, "nco:contributor");
@@ -365,11 +524,19 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", xd->publisher);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
+
 		tracker_sparql_builder_subject_iri (preupdate, uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nco:Contact");
 		tracker_sparql_builder_predicate (preupdate, "nco:fullname");
 		tracker_sparql_builder_object_unvalidated (preupdate, xd->publisher);
+
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 
 		tracker_sparql_builder_predicate (metadata, "nco:creator");
@@ -423,12 +590,17 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 
 		if (xd->address || xd->state || xd->country || xd->city)  {
 			gchar *addruri;
+
 			addruri = tracker_sparql_get_uuid_urn ();
 
 			tracker_sparql_builder_predicate (metadata, "slo:postalAddress");
-			tracker_sparql_builder_object_iri (metadata, addruri);			
-			
+			tracker_sparql_builder_object_iri (metadata, addruri);
+
 			tracker_sparql_builder_insert_open (preupdate, NULL);
+			if (graph) {
+				tracker_sparql_builder_graph_open (preupdate, graph);
+			}
+
 			tracker_sparql_builder_subject_iri (preupdate, addruri);
 
 			g_free (addruri);
@@ -456,6 +628,9 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 			  tracker_sparql_builder_object_unvalidated (preupdate, xd->country);
 			}
 
+			if (graph) {
+				tracker_sparql_builder_graph_close (preupdate);
+			}
 			tracker_sparql_builder_insert_close (preupdate);
 		}
 
@@ -498,6 +673,10 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		tracker_sparql_builder_object_double (metadata, value);
 	}
 
+	if (xd->regions) {
+		tracker_xmp_apply_regions (preupdate, metadata, graph, xd);
+	}
+
 	for (i = 0; i < keywords->len; i++) {
 		gchar *p, *escaped, *var;
 
@@ -506,11 +685,26 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		var = g_strdup_printf ("tag%d", i + 1);
 
 		/* ensure tag with specified label exists */
+		tracker_sparql_builder_append (preupdate, "INSERT { ");
+
+		if (graph) {
+			tracker_sparql_builder_append (preupdate, "GRAPH <");
+			tracker_sparql_builder_append (preupdate, graph);
+			tracker_sparql_builder_append (preupdate, "> { ");
+		}
+
 		tracker_sparql_builder_append (preupdate,
-		                               "INSERT { _:tag a nao:Tag ; nao:prefLabel \"");
+		                               "_:tag a nao:Tag ; nao:prefLabel \"");
 		tracker_sparql_builder_append (preupdate, escaped);
+		tracker_sparql_builder_append (preupdate, "\"");
+
+		if (graph) {
+			tracker_sparql_builder_append (preupdate, " } ");
+		}
+
+		tracker_sparql_builder_append (preupdate, " }\n");
 		tracker_sparql_builder_append (preupdate,
-		                               "\" }\nWHERE { FILTER (NOT EXISTS { "
+		                               "WHERE { FILTER (NOT EXISTS { "
 		                               "?tag a nao:Tag ; nao:prefLabel \"");
 		tracker_sparql_builder_append (preupdate, escaped);
 		tracker_sparql_builder_append (preupdate,
@@ -540,7 +734,7 @@ guess_dlna_profile (gint          depth,
                     const gchar **dlna_profile,
                     const gchar **dlna_mimetype)
 {
-	gchar *profile = NULL;
+	const gchar *profile = NULL;
 
 	if (dlna_profile) {
 		*dlna_profile = NULL;
@@ -576,11 +770,7 @@ guess_dlna_profile (gint          depth,
 }
 
 G_MODULE_EXPORT gboolean
-tracker_extract_get_metadata (const gchar          *uri,
-                              const gchar          *mimetype,
-                              TrackerSparqlBuilder *preupdate,
-                              TrackerSparqlBuilder *metadata,
-                              GString              *where)
+tracker_extract_get_metadata (TrackerExtractInfo *info)
 {
 	goffset size;
 	FILE *f;
@@ -592,17 +782,25 @@ tracker_extract_get_metadata (const gchar          *uri,
 	png_uint_32 width, height;
 	gint bit_depth, color_type;
 	gint interlace_type, compression_type, filter_type;
-	const gchar *dlna_profile;
-	gchar *filename;
+	const gchar *dlna_profile, *graph;
+	TrackerSparqlBuilder *preupdate, *metadata;
+	gchar *filename, *uri;
+	GString *where;
+	GFile *file;
 
-	filename = g_filename_from_uri (uri, NULL, NULL);
+	file = tracker_extract_info_get_file (info);
+	filename = g_file_get_path (file);
 	size = tracker_file_get_size (filename);
 
+	preupdate = tracker_extract_info_get_preupdate_builder (info);
+	metadata = tracker_extract_info_get_metadata_builder (info);
+	graph = tracker_extract_info_get_graph (info);
+
 	if (size < 64) {
 		return FALSE;
 	}
 
-	f = tracker_file_open (filename, "r", FALSE);
+	f = tracker_file_open (filename);
 	g_free (filename);
 
 	if (!f) {
@@ -679,7 +877,13 @@ tracker_extract_get_metadata (const gchar          *uri,
 	tracker_sparql_builder_object (metadata, "nfo:Image");
 	tracker_sparql_builder_object (metadata, "nmm:Photo");
 
-	read_metadata (preupdate, metadata, where, png_ptr, info_ptr, end_ptr, uri);
+	uri = g_file_get_uri (file);
+	where = g_string_new ("");
+
+	read_metadata (preupdate, metadata, where, png_ptr, info_ptr, end_ptr, uri, graph);
+	tracker_extract_info_set_where_clause (info, where->str);
+	g_string_free (where, TRUE);
+	g_free (uri);
 
 	tracker_sparql_builder_predicate (metadata, "nfo:width");
 	tracker_sparql_builder_object_int64 (metadata, width);
